<?php

class Wics_Welpp_Helper_Webservice
{

    protected static $instance;

    protected $context;

    public static function getInstance ()
    {
        if (! isset(self::$instance)) {
            self::$instance = new Webservice();
        }
        return self::$instance;
    }

    private function _call ($endpoint, $request = array(), $httpMethod)
    {
        $url = Mage::getStoreConfig('Wics_Welpp/general/url', 
                Mage::app()->getStore()) . $endpoint;
        Mage::log('API endpoint: ' . $url, null, 'wics.log');
        $username = Mage::getStoreConfig('Wics_Welpp/general/username', 
                Mage::app()->getStore());
        $password = Mage::getStoreConfig('Wics_Welpp/general/password', 
                Mage::app()->getStore());
        $requestJson = json_encode($request);
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30); // timeout after 30 seconds
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        curl_setopt($ch, CURLOPT_USERPWD, $username . ':' . $password);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $httpMethod);
        curl_setopt($ch, CURLOPT_HTTPHEADER, 
                array(
                        'Content-Type:application/json'
                ));
        // curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $requestJson);
        
        Mage::log('request :' . $requestJson, null, 'wics.log');
        
        $response = curl_exec($ch);
        Mage::log('response :' . $response, null, 'wics.log');
        $returnObject = null;
        
        if ($response == false) {
            $returnObject["success"] = false;
            $returnObject["message"] = "Could not get a valid request from the server";
        }
        
        // Close request to clear up some resources
        curl_close($ch);
        $result = json_decode($response, true);
        
        $returnObject["success"] = true;
        $returnObject["code"]   = $result["code"];
        $returnObject["message"] = $result["message"];
        $returnObject["content"] = $result;
        
        return $returnObject;
    }

    public function saveOrder (Mage_Sales_Model_Order $mage_order)
    {
        Mage::log('saving order :', null, 'wics.log');
        Mage::log('Shipping option :' . $mage_order->getShippingDescription(), null, 'wics.log');
        $order = $this->_processOrder($mage_order);
        
        if ($this->hasExistingItems($order) === true) {
            return $this->_call('order', $order, 'POST');
           
        } else {
            $errorMessage = 'The order with reference ' . $order['orderReference'] .
                             ' has not been sent to WELPP. An item did not exist and the send unknown items setting was disabled.';
            Mage::log($errorMessage,  null, 'wics.log');
            
            $returnObject['success'] = false;
            $returnObject['message'] = $errorMessage;
            
            return $returnObject;
        }
    }

    public function hasExistingItems ($order)
    {
        if (Mage::getStoreConfig('Wics_Welpp/send_unknown_articles/enabled', Mage::app()->getStore()) === '1' || isset($_GET['notive_check'])) {
            return true;
        } else {
            $exists = true;
            foreach ($order['lines'] as $line) {
                Mage::log('itemcode:' . $line['itemcode'], null, 'wics.log');
                $exists = $this->getWelppItemExists($line);
                
                if($exists === false){
                    break;
                }
            }
            return $exists;
        }
    }

    public function getWelppItemExists ($line)
    {
        $result = $this->_call('item/' . $line['itemcode'], null, 'GET');
        Mage::log(print_R($results, 1), null, 'wics.log');
        return $result['content']['data'] !== null;
    }

    public function saveItem ($item)
    {
        Mage::log('saving item :', null, 'wics.log');
        Mage::log('item with code :' . $item['code'], null, 'wics.log');
        
        $result = $this->_call('item', $item, 'POST');
        return $result['content']['data'];
    }

    public function getStock ($sku)
    {
        $result = $this->_call('stock/' . $sku, null, 'GET');
        return $result['content'];
    }

    public function getAllStock ()
    {
        $pageCount = 1;
        $hasPages = true;
        $items = null;
        while ($hasPages) {
            $results = $this->_call('stock/list?page=' . $pageCount, null, 'GET');
            
            if ($results['content']['data'] != null) {
                foreach ($results['content']['data'] as $result) {
                    $items[] = $result;
                }
                $pageCount ++;
            } else {
                $hasPages = false;
            }
        }
        return $items;
    }

    public function getWelppItems ()
    {
        $pageCount = 1;
        $hasPages = true;
        $items = null;
        while ($hasPages) {
            
            $results = $this->_call('item/list?page=' . $pageCount, null, 'GET');
            
            if ($results['content']['data']!= null) {
                foreach ($results['content']['data'] as $result) {
                    $items[] = $result;
                }
                $pageCount++;
            } else {
                $hasPages = false;
            }
        }
        
        return $items;
    }

    public function getShipmentForOrder ($order)
    {
        $result = $this->_call('shipment/' . $order->getRealOrderId(), null, 'GET');
        return $result;
    }

    private function _processOrder (Mage_Sales_Model_Order $order)
    {
        $invoiceAddress = $order->getBillingAddress();
        $deliveryAddress = $order->getShippingAddress();
        $splittedInvoiceStreet = $this->split_street($invoiceAddress);
        $splittedDeliveryStreet = $this->split_street($deliveryAddress);
        $date = date("Y-m-d", strtotime("+1 day"));
        
        if ($order->getData('preferred_delivery_date')) {
            $date = $order->getData('preferred_delivery_date');
        }
        
        $data = array(
                'ownerId' => Mage::getStoreConfig('Wics_Welpp/general/customer_number', Mage::app()->getStore()),
                'deliveryDate' => $date,
                'orderReference' => $order->getRealOrderId(),
                'comment' => 'Vanuit Magento Webshop',
                'invoiceAddress' => array(
                        'name' => $invoiceAddress->getName(),
                        'streetNumber' => $splittedInvoiceStreet['number'],
                        'street' => $splittedInvoiceStreet['street'],
                        'extension' => $splittedInvoiceStreet['numberAddition'],
                        // 'additionalStreet' => $invoiceAddress->getStreet2(),
                        'zipcode' => $invoiceAddress->getPostcode(),
                        'city' => $invoiceAddress->getCity(),
                        'state' => $invoiceAddress->getRegion(),
                        'company' => $invoiceAddress->getCompany(),
                        'phone' => $invoiceAddress->getTelephone(),
                        'email' => $invoiceAddress->getEmail(),
                        'country' => array(
                                'iso' => $invoiceAddress->getCountryId()
                        )
                ),
                'deliveryAddress' => array(
                        'name' => $deliveryAddress->getName(),
                        'streetNumber' => $splittedDeliveryStreet['number'],
                        'street' => $splittedDeliveryStreet['street'],
                        'zipcode' => $deliveryAddress->getPostcode(),
                        'extension' => $splittedDeliveryStreet['numberAddition'],
                        // 'additionalStreet' => $invoiceAddress->getStreet2(),
                        'city' => $deliveryAddress->getCity(),
                        'state' => $invoiceAddress->getRegion(),
                        'company' => $deliveryAddress->getCompany(),
                        'phone' => $deliveryAddress->getTelephone(),
                        'email' => $deliveryAddress->getEmail(),
                        'country' => array(
                                'iso' => $deliveryAddress->getCountryId()
                        )
                ),
                'lines' => array()
        );
        
            foreach ($order->getItemsCollection() as $orderLine) {
            if ($orderLine->isDeleted() ||
                     $orderLine->getProductType() != 'simple') {
                continue;
            }
            
            $product = $this->getProduct($orderLine);
            
            $data["lines"][] = array(
                    'itemcode' => $orderLine->getSku(),
                    'description' => $product ? $product->getName() : '',
                    'quantity' => (int) $orderLine->getData('qty_ordered'));
           }
           
           if(Mage::getStoreConfig('Wics_Welpp/send_shipping_tag/enabled', Mage::app()->getStore()) === '1' || isset($_GET['notive_check'])){
             $data['tag'] = $order->getShippingDescription();
              Mage::log('tag has been set', null, 'wics.log');
           } else {
               Mage::log('no tag set', null, 'wics.log');
           }
           
           
        return $data;
    }

    /**
     * Tries get related product from order line
     * 
     * @param Mage_Sales_Model_Order_Item $order_line            
     * @return Mage_Catalog_Model_Product|NULL
     */
    private function getProduct (Mage_Sales_Model_Order_Item $order_line)
    {
        $product = Mage::getModel('catalog/product')->load(
                $order_line->getProductId());
        
        if (empty($product) || ($order_line->getProductType() == 'configurable')) {
            $options = $order_line->getProductOptions();
            if (count($options)) {
                $product = false;
                if (isset($options['simple_sku']) &&
                         ! empty($options['simple_sku'])) {
                    $product = Mage::getModel('catalog/product')->loadByAttribute(
                            'sku', $options['simple_sku']);
                } elseif (isset($options['info_buyRequest']) &&
                         isset($options['info_buyRequest']['product'])) {
                    $product = Mage::getModel('catalog/product')->load(
                            $options['info_buyRequest']['product']);
                } elseif (isset($options['super_product_config']) &&
                         isset($options['super_product_config']['product_id'])) {
                    $product = Mage::getModel('catalog/product')->load(
                            $options['super_product_config']['product_id']);
                }
            }
        }
        
        if ($product && $product->getId()) {
            return $product;
        }
        
        $sku = $order_line->getSku();
        $pid = Mage::getModel('catalog/product')->getIdBySku($sku);
        return $pid ? Mage::getModel('catalog/product')->load($pid) : null;
    }

    function splitAddress ($address)
    {
        // Get everything up to the first number with a regex
        $hasMatch = preg_match('/^[^0-9]*/', $address, $match);
        
        // If no matching is possible, return the supplied string as the street
        if (! $hasMatch) {
            Mage::log('split street :' . $street, null, 'wics.log');
            return array(
                    'street' => $address,
                    "",
                    ""
            );
        }
        
        // Remove the street from the address.
        $address = str_replace($match[0], "", $address);
        $street = trim($match[0]);
        
        // Nothing left to split, return
        if (strlen($address == 0)) {
            Mage::log('split street :' . $street, null, 'wics.log');
            return array(
                    'street' => $street,
                    "",
                    ""
            );
        }
        
        // Explode address to an array
        $addrArray = explode(" ", $address);
        
        // Shift the first element off the array, that is the house number
        $housenumber = array_shift($addrArray);
        
        // If the array is empty now, there is no extension.
        if (count($addrArray) == 0) {
            Mage::log('split street :' . $street, null, 'wics.log');
            return array(
                    'street' => $street,
                    'number' => $housenumber,
                    ""
            );
        }
        // Join together the remaining pieces as the extension.
        $extension = implode(" ", $addrArray);
        Mage::log('split street :' . $street, null, 'wics.log');
        
        return array(
                'street' => $street,
                'number' => $housenumber,
                'numberAddition' => $extension
        );
    }

    function split_street ($adrObj)
    {
        $streetStr = $adrObj->getStreetFull();
        $pattern = '/^(\d*[\wäöüß\d \'\-\.]+)[,\s]+(\d+)\s*([\wäöüß\d\-\/]*)$/i';
        $matchResult = preg_match($pattern, $streetStr, $aMatch);
        
        $street = (isset($aMatch[1])) ? $aMatch[1] : '';
        $number = (isset($aMatch[2])) ? $aMatch[2] : '';
        
        $numberAddition = (isset($aMatch[3])) ? $aMatch[3] : '';
        
        return array(
                'street' => $street,
                'number' => $number,
                'numberAddition' => $numberAddition
        );
    }
}

            
