<?php

class Wics_Welpp_Model_Observers_Order extends Wics_Welpp_Model_Observers_Order_Abstract {
    const WICS_ORDER_STATUS_CODE_ERROR = 'notive_wics_error';
    const WICS_ORDER_STATUS_CODE_OK = 'notive_wics_sent';

    const WICS_SAVED_MARK = 'notive_wics_order_saved';

    private $_enabled,
            $_status_send;

    public function _construct()
    {
        parent::_construct();

        $this->_enabled =
            Mage::getStoreConfig('Wics_Welpp/order/enabled', Mage::app()->getStore()) === '1';
        $this->_status_send =
            explode(',', Mage::getStoreConfig('Wics_Welpp/order/status_send', Mage::app()->getStore()));
    }

    /**
     * Process order
     * @param Varien_Event_Observer|Mage_Sales_Model_Order $event
     * @param bool $manual this is manual call, not from event handler
     */
    public function sales_order_save_after($event, $manual = false)
    {
        $helper_webservice = Mage::getSingleton('Wics_Welpp_Helper_Webservice');

        if (!$manual && !$this->_enabled) {
            return;
        }

        if (!$manual) {
            $get_order = version_compare(Mage::getVersion(), '1.4', '<') === true ? 'getOrder' : 'getDataObject';

            if (!$event || !$event->$get_order()) {
                return;
            }

            $order = $event->$get_order();
        } else {
            $order = $event;
        }

        if ($order->getData(self::WICS_SAVED_MARK)) {
            return;
        }

        if ($this->shouldSendOrder($order)) {
            $order->setData(self::WICS_SAVED_MARK, true);

            $result = $helper_webservice->saveOrder($order);            

    // Write the contents back to the file

            if ($result !== false && !is_null($result) && isset($result['success']) && $result['success']) {
                $this->setStateSent($order);
            } else {
                $msg = '';
                if ($result !== false && !is_null($result) && isset($result['error'])) {
                    $msg = $result['error'];
                }
                $this->setStateNotSent($order, $msg);
            }
        }
    }

    /**
     * Should we send the order to WICS
     * @param  Mage_Sales_Model_Order $order
     * @return bool
     */
    protected function shouldSendOrder(Mage_Sales_Model_Order $order)
    {
        if (in_array($order->getStatus(), $this->_status_send)) {
            $comments = $order->getStatusHistoryCollection(true);
            $should_send = true;
            foreach ($comments as $comment) {
                if (preg_match('/^wics_notive_sent/Usi', $comment->getStatus())) {
                    $should_send = false;
                }
            }
            if ($should_send !== false) {
                return true;
            }
        }
        return false;
    }

    /**
     * Mark order as not sent
     *
     * @param Mage_Sales_Model_Order $order
     * @throws Mage_Core_Exception
     */
    public function setStateNotSent(Mage_Sales_Model_Order $order, $msg = '')
    {
        if (!$order->canHold()) {
            Mage::throwException($this->__('Error setting hold status.'));
        }
        $order->setHoldBeforeState($order->getState());
        $order->setHoldBeforeStatus($order->getStatus());
        $order->setState(
            Mage_Sales_Model_Order::STATE_HOLDED,
            self::WICS_ORDER_STATUS_CODE_ERROR,
            'Order can\'t be sent to Welpp' . ($msg ? ".<br/>Error: {$msg}" : '.')
        )->save();
    }

    /**
     * Mark order as sent
     *
     * @param Mage_Sales_Model_Order $order
     * @throws Mage_Core_Exception
     */
    public function setStateSent(Mage_Sales_Model_Order $order)
    {
        if ($order->getHoldBeforeState() && !$order->canUnhold()) {
            Mage::throwException($this->__('Error removing hold status.'));
        }
        $state = $order->getHoldBeforeState() ? $order->getHoldBeforeState() : $order->getState();
        $order->setState($state, self::WICS_ORDER_STATUS_CODE_OK, 'Order was sent to Welpp.')->save();
        $order->setHoldBeforeState(null);
        $order->setHoldBeforeStatus(null);
    }

    /**
     * Mark order as pending
     *
     * @param Mage_Sales_Model_Order $order
     * @throws Mage_Core_Exception
     */
    public function setStatePending(Mage_Sales_Model_Order $order)
    {
        if ($order->getHoldBeforeState() && !$order->canUnhold()) {
            Mage::throwException($this->__('Error removing hold status.'));
        }
        $state = $order->getHoldBeforeState() ? $order->getHoldBeforeState() : $order->getState();
        $order->setState(Mage_Sales_Model_Order::STATE_NEW, true)->save();
        $order->setHoldBeforeState(null);
        $order->setHoldBeforeStatus(null);
    }

}
